{% extends 'base.html' %}

{% block header %}
  <h1>{% block title %}Vehicles{% endblock %}</h1>
{% endblock %}

{% block content %}
<table style="width: 100%;">
  <!-- Table headers not necessary-->
  {% for vehicle in vehicles %}
  <tr>
      <td style="width: 400px"><img src="{{ url_for('status.vehicleImg', vin=vehicle.vin.value) }}" class="status"/></td>
      <td>{{vehicle.nickname.value}}</td>
      <td>{{vehicle.model.value}}</td>
      <td><a href="{{ url_for('settings.vehicle', vin=vehicle.vin.value) }}"><img src="{{ url_for('static', filename='icons/settings.png') }}" class="icon"/></td></a></td>
  </tr>
  {% endfor %}
</table>
{% endblock %}